@@ -53,8 +53,8 @@ static void *__init check_function_address(void *ptr, char *symbol)
  *
  * Returns return value from kernel_read().
  */
-static int __init my_kernel_read(struct file *file, unsigned long offset,
-				    char *addr, unsigned long count)
+static inline int my_kernel_read(struct file *file, unsigned long offset,
+				 char *addr, unsigned long count)
 {
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 8)
 	/*
@@ -90,6 +90,16 @@ static int __init my_kernel_read(struct file *file, unsigned long offset,
  */
 static void *__init probe_find_symbol(const char *keyline)
 {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
+	char buf[128] = { };
+	char *cp;
+
+	strncpy(buf, keyline + 1, sizeof(buf) - 1);
+	cp = strchr(buf, '\n');
+	if (cp)
+		*cp = '\0';
+	return (void *) kallsyms_lookup_name(buf);
+#else
 	struct file *file = NULL;
 	char *buf;
 	unsigned long entry = 0;
@@ -177,6 +187,7 @@ static void *__init probe_find_symbol(const char *keyline)
 	filp_close(file, NULL);
 out:
 	return (void *) entry;
+#endif
 }
 
 #endif
