@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.8.5   2015/11/11
+ * Version: 1.8.6+   2019/12/25
  */
 
 #include "internal.h"
@@ -15,8 +15,6 @@
 
 /***** SECTION1: Constants definition *****/
 
-#define SOCKFS_MAGIC 0x534F434B
-
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
 #define s_fs_info u.generic_sbp
 #endif
@@ -37,30 +35,10 @@ static char *ccs_get_dentry_path(struct dentry *dentry, char * const buffer,
 				 const int buflen);
 static char *ccs_get_local_path(struct dentry *dentry, char * const buffer,
 				const int buflen);
-static char *ccs_get_socket_name(const struct path *path, char * const buffer,
-				 const int buflen);
 static int ccs_const_part_length(const char *filename);
 
 /***** SECTION4: Standalone functions section *****/
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)
-
-/**
- * SOCKET_I - Get "struct socket".
- *
- * @inode: Pointer to "struct inode".
- *
- * Returns pointer to "struct socket".
- *
- * This is for compatibility with older kernels.
- */
-static inline struct socket *SOCKET_I(struct inode *inode)
-{
-	return inode->i_sock ? &inode->u.socket_i : NULL;
-}
-
-#endif
-
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 37)
 
 /**
@@ -431,9 +409,12 @@ static char *ccs_get_local_path(struct dentry *dentry, char * const buffer,
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3, 17, 0)
 		if (inode->i_op && !inode->i_op->rename)
 			goto prepend_filesystem_name;
-#else
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(4, 9, 0)
 		if (!inode->i_op->rename && !inode->i_op->rename2)
 			goto prepend_filesystem_name;
+#else
+		if (!inode->i_op->rename)
+			goto prepend_filesystem_name;
 #endif
 	}
 	/* Prepend device name. */
@@ -467,33 +448,6 @@ out:
 	return ERR_PTR(-ENOMEM);
 }
 
-/**
- * ccs_get_socket_name - Get the name of a socket.
- *
- * @path:   Pointer to "struct path".
- * @buffer: Pointer to buffer to return value in.
- * @buflen: Sizeof @buffer.
- *
- * Returns the buffer.
- */
-static char *ccs_get_socket_name(const struct path *path, char * const buffer,
-				 const int buflen)
-{
-	struct inode *inode = d_backing_inode(path->dentry);
-	struct socket *sock = inode ? SOCKET_I(inode) : NULL;
-	struct sock *sk = sock ? sock->sk : NULL;
-	if (sk) {
-		snprintf(buffer, buflen, "socket:[family=%u:type=%u:"
-			 "protocol=%u]", sk->sk_family, sk->sk_type,
-			 sk->sk_protocol);
-	} else {
-		snprintf(buffer, buflen, "socket:[unknown]");
-	}
-	return buffer;
-}
-
-#define SOCKFS_MAGIC 0x534F434B
-
 /**
  * ccs_realpath - Returns realpath(3) of the given pathname but ignores chroot'ed root.
  *
@@ -524,13 +478,8 @@ char *ccs_realpath(const struct path *path)
 			break;
 		/* To make sure that pos is '\0' terminated. */
 		buf[buf_len - 1] = '\0';
-		/* Get better name for socket. */
-		if (sb->s_magic == SOCKFS_MAGIC) {
-			pos = ccs_get_socket_name(path, buf, buf_len - 1);
-			goto encode;
-		}
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 22)
-		/* For "pipe:[\$]". */
+		/* For "pipe:[\$]" and "socket:[\$]". */
 		if (dentry->d_op && dentry->d_op->d_dname) {
 			pos = dentry->d_op->d_dname(dentry, buf, buf_len - 1);
 			goto encode;
@@ -538,12 +487,14 @@ char *ccs_realpath(const struct path *path)
 #endif
 		inode = d_backing_inode(sb->s_root);
 		/*
-		 * Use local name for "filesystems without rename() operation"
-		 * or "path without vfsmount" or "absolute name is unavailable"
-		 * cases.
+		 * Use local name for "filesystems without rename() operation
+		 * and device file" or "path without vfsmount" or "absolute
+		 * name is unavailable" cases.
 		 */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(3, 17, 0)
-		if (!path->mnt || (inode->i_op && !inode->i_op->rename))
+		if (!path->mnt ||
+		    (inode->i_op && !inode->i_op->rename &&
+		     !(sb->s_type->fs_flags & FS_REQUIRES_DEV)))
 			pos = ERR_PTR(-EINVAL);
 		else {
 			/* Get absolute name for the rest. */
@@ -554,9 +505,18 @@ char *ccs_realpath(const struct path *path)
 		if (pos == ERR_PTR(-EINVAL))
 			pos = ccs_get_local_path(path->dentry, buf,
 						 buf_len - 1);
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(4, 9, 0)
+		if (!path->mnt ||
+		    (!inode->i_op->rename && !inode->i_op->rename2 &&
+		     !(sb->s_type->fs_flags & FS_REQUIRES_DEV)))
+			pos = ccs_get_local_path(path->dentry, buf,
+						 buf_len - 1);
+		else
+			pos = ccs_get_absolute_path(path, buf, buf_len - 1);
 #else
 		if (!path->mnt ||
-		    (!inode->i_op->rename && !inode->i_op->rename2))
+		    (!inode->i_op->rename &&
+		     !(sb->s_type->fs_flags & FS_REQUIRES_DEV)))
 			pos = ccs_get_local_path(path->dentry, buf,
 						 buf_len - 1);
 		else
@@ -701,7 +661,11 @@ void ccs_fill_path_info(struct ccs_path_info *ptr)
 	ptr->const_len = ccs_const_part_length(name);
 	ptr->is_dir = len && (name[len - 1] == '/');
 	ptr->is_patterned = (ptr->const_len < len);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 8, 0)
+	ptr->hash = full_name_hash(NULL, name, len);
+#else
 	ptr->hash = full_name_hash(name, len);
+#endif
 }
 
 /**
